//************************************************************
//  University of Patras
//	Department of Electrical Engineering & Computer Technology
//
//	Course: Parallel Programming in AI
//	Week 05: OpenMp
//
//  Description: Simple examples  in openMP
//  	pi07.c: Estimation of pi using monteCarlo method
//	Author: Evangelos Dermatas
//	v0.1
/*
*********************************************************************
gcc -v:	gcc version 7.4.0 (Ubuntu 7.4.0-1ubuntu1~18.04.1)
OS: 	Linux Ubuntu 18.04
CPU:	8-core, Intel(R) Core(TM) i7-2630QM CPU @ 2.00GHz, 3991 BogoMips
*********************************************************************
Compiling:	gcc pi07.c -o pi07 -fopenmp -O3
Executing:	./pi07

pi= 3.14155005118822128
real	0m2.050s
user	0m16.267s
sys	0m0.001s

Previous version pi03
pi= 3.14159265558978928

real	0m2.045s
user	0m16.241s
sys	0m0.004s


*/
//************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <omp.h>
#include <math.h>


#define N 1000000000

unsigned int seed = 666999666 ;  // random number seed

// **************************************************************
unsigned int randUint( void )
{
#pragma omp threadprivate(seed)
	seed = seed * 1103515245 + 12345;
return seed ;
}


// **************************************************************
int main(void)
{
	double sum = 0 ;
	double step ;

#pragma omp threadprivate(seed)

	step = 1.0/N ;
	double nrm = 1.0/UINT_MAX ;

#pragma omp parallel for simd reduction(+:sum)
	for( int i = 0 ; i < N ; i++ ) {
		double x = randUint()*nrm ;
		sum += sqrt( 1.0 - x*x ) ;
		}

	printf( "pi=%20.17lf\n", 4.0 * sum * step ) ;
	return 0 ;
}
